@@ -5,17 +5,18 @@ import numpy as np
 import pandas as pd
 import ruamel.yaml as yaml
 
-from petsc4py import PETSc
 from operator import itemgetter
 from scipy.interpolate import interp1d
 
 petsc4py.init(sys.argv)
-MPIrank = PETSc.COMM_WORLD.Get_rank()
+MPIrank = petsc4py.PETSc.COMM_WORLD.Get_rank()
 
 
 class ReadYaml(object):
     """
-    Reading simulation input file.
+    Class for reading simulation input file and initialising model parameters.
+
+    Definition of input parameters is provided in the `User Documentation <https://gospl.readthedocs.io/en/latest/inputfile.html>`_
     """
 
     def __init__(self, filename):
@@ -26,7 +27,7 @@ class ReadYaml(object):
         """
 
         if self.showlog:
-            self.log = PETSc.Log()
+            self.log = petsc4py.PETSc.Log()
             self.log.begin()
 
         # Check input file exists
@@ -55,10 +56,21 @@ class ReadYaml(object):
         self._readSealevel()
         self._readTectonic()
         self._readRain()
+        self._readCompaction()
+
+        if self.raindata is not None:
+            if self.rStep > 0:
+                self.raindata = self.raindata[
+                    self.raindata["start"] >= self.tStart + self.rStep * self.tout
+                ]
+                self.raindata.reset_index(drop=True, inplace=True)
+                self.rainNb = len(self.raindata)
+
         self._readBackwardPaleo()
         self._readOut()
         self._readForcePaleo()
 
+        self.radius = 6378137.0
         self.gravity = 9.81
         self.tNow = self.tStart
         self.saveTime = self.tNow
@@ -121,6 +133,43 @@ class ReadYaml(object):
         except KeyError:
             self.backward = False
 
+        try:
+            self.interp = domainDict["interp"]
+        except KeyError:
+            self.interp = 1
+
+        self._extraDomain()
+
+        return
+
+    def _extraDomain(self):
+        """
+        Read domain additional information.
+        """
+
+        domainDict = self.input["domain"]
+
+        try:
+            self.overlap = domainDict["overlap"]
+        except KeyError:
+            self.overlap = 1
+
+        try:
+            dataFile = domainDict["nperodep"]
+            self.dataFile = dataFile + ".npz"
+            with open(self.dataFile) as dataFile:
+                dataFile.close()
+        except KeyError:
+            self.dataFile = None
+
+        try:
+            strataFile = domainDict["npstrata"]
+            self.strataFile = strataFile + ".npz"
+            with open(self.strataFile) as strataFile:
+                strataFile.close()
+        except KeyError:
+            self.strataFile = None
+
         return
 
     def _readTime(self):
@@ -210,6 +259,41 @@ class ReadYaml(object):
         except KeyError:
             self.strat = 0
 
+        if self.tout < self.tecStep:
+            self.tecStep = self.tout
+            print(
+                "Output time interval and tectonic forcing time step \
+                 have been adjusted to match each others.",
+                flush=True,
+            )
+
+        if self.tout < self.strat:
+            self.strat = self.tout
+            print(
+                "Output time interval and stratal time step \
+                 have been adjusted to match each others.",
+                flush=True,
+            )
+
+        if self.tecStep > 0:
+            if self.tout % self.tecStep != 0:
+                print(
+                    "When declaring tectonic time interval, the value should be divisible by the output time interval.",
+                    flush=True,
+                )
+                raise ValueError("Tectonic time interval definition is wrong!")
+
+        if self.strat > 0:
+            if self.tout % self.strat != 0:
+                print(
+                    "When declaring stratal time interval, the value should be divisible by the output time interval.",
+                    flush=True,
+                )
+                raise ValueError("Stratal time interval definition is wrong!")
+            self.stratNb = int((self.tEnd - self.tStart) / self.strat) + 1
+        else:
+            self.stratNb = 0
+
         return
 
     def _readSPL(self):
@@ -229,21 +313,38 @@ class ReadYaml(object):
                 raise ValueError("Surface Process Model: Kb coefficient not found.")
             try:
                 self.frac_fine = splDict["Ff"]
+                if self.frac_fine < 0.05:
+                    self.frac_fine = 0.05
+                    if MPIrank == 0:
+                        print(
+                            "The fraction of fine has been reset to 5%, which is minimal authorised value.",
+                            flush=True,
+                        )
             except KeyError:
-                self.frac_fine = 0.0
+                self.frac_fine = 0.05
             try:
-                # `wgth` is the percentage of upstream sediment flux
-                # that will be deposited on each cell...
-                self.wgth = splDict["wgth"]
-                if self.wgth >= 1.0:
-                    self.wgth = 0.999
+                self.fillmax = splDict["hfill"]
             except KeyError:
-                self.wgth = 0.0
+                self.fillmax = 100.0
+            try:
+                self.coeffd = splDict["d"]
+            except KeyError:
+                self.coeffd = 0.0
+            # try:
+            #     # `wght` is the percentage of upstream sediment flux
+            #     # that will be deposited on each cell...
+            #     self.wght = splDict["wght"]
+            #     if self.wght >= 1.0:
+            #         self.wght = 0.999
+            # except KeyError:
+            #     self.wght = 0.0
 
         except KeyError:
             self.K = 1.0e-12
-            self.wgth = 0.0
-            self.frac_fine = 0.0
+            self.fillmax = 100.0
+            # self.wght = 0.0
+            self.frac_fine = 0.05
+            self.coeffd = 0.0
 
         return
 
@@ -254,6 +355,11 @@ class ReadYaml(object):
 
         try:
             hillDict = self.input["diffusion"]
+
+            try:
+                self.shelfslope = hillDict["shelfslope"]
+            except KeyError:
+                self.shelfslope = False
             try:
                 self.Cda = hillDict["hillslopeKa"]
             except KeyError:
@@ -275,13 +381,19 @@ class ReadYaml(object):
                     "Hillslope: Cd coefficient for marine environment not found."
                 )
             try:
-                self.sedimentK = hillDict["sedimentK"]
+                self.sedimentK = hillDict["sedK"]
             except KeyError:
-                self.sedimentK = 10.0
+                self.sedimentK = 10000.0
+            try:
+                self.sedimentKf = hillDict["sedKf"]
+            except KeyError:
+                self.sedimentKf = 20000.0
         except KeyError:
+            self.shelfslope = False
             self.Cda = 0.0
             self.Cdm = 0.0
-            self.sedimentK = 10.0
+            self.sedimentK = 10000.0
+            self.sedimentKf = 20000.0
 
         return
 
@@ -510,7 +622,12 @@ class ReadYaml(object):
                     [pd.DataFrame(tmpTec, columns=["start", "tMap", "zMap"]), tecdata],
                     ignore_index=True,
                 )
-            self.tecdata = tecdata
+            self.tecdata = tecdata[tecdata["start"] >= self.tStart]
+            if self.rStep > 0:
+                self.tecdata = tecdata[
+                    tecdata["start"] >= self.tStart + self.rStep * self.tout
+                ]
+            self.tecdata.reset_index(drop=True, inplace=True)
 
         except KeyError:
             self.tecdata = None
@@ -545,7 +662,8 @@ class ReadYaml(object):
         tmpRain = []
         if rMap is None:
             tmpRain.insert(
-                0, {"start": rStart, "rUni": rUniform, "rMap": None, "rKey": None},
+                0,
+                {"start": rStart, "rUni": rUniform, "rMap": None, "rKey": None},
             )
         else:
             tmpRain.insert(
@@ -643,7 +761,7 @@ class ReadYaml(object):
                             "Field name for rainfall is not defined correctly or does not exist!"
                         )
 
-                    raindata = self._defineRain(k, rStart, rMap, rUniform, raindata)
+                raindata = self._defineRain(k, rStart, rMap, rUniform, raindata)
 
             if raindata["start"][0] > self.tStart:
                 tmpRain = []
@@ -659,7 +777,9 @@ class ReadYaml(object):
                     ],
                     ignore_index=True,
                 )
-            self.raindata = raindata
+            self.raindata = raindata[raindata["start"] >= self.tStart]
+            self.raindata.reset_index(drop=True, inplace=True)
+            self.rainNb = len(self.raindata)
 
         except KeyError:
             self.raindata = None
@@ -708,8 +828,14 @@ class ReadYaml(object):
                         ignore_index=True,
                     )
 
-            self.paleodata = paleodata
-            self.paleoNb = len(paleodata)
+            self.paleodata = paleodata[paleodata["time"] >= self.tStart]
+            if self.rStep > 0:
+                self.paleodata = paleodata[
+                    paleodata["start"] >= self.tStart + self.rStep * self.tout
+                ]
+
+            self.paleodata.reset_index(drop=True, inplace=True)
+            self.paleoNb = len(self.paleodata)
 
         except KeyError:
             self.paleodata = None
@@ -730,31 +856,33 @@ class ReadYaml(object):
                 if not os.path.exists(self.forceDir):
                     print("Forcing paleo directory does not exist!", flush=True)
                     raise ValueError("Forcing paleo directory does not exist!")
-                if self.tout > self.tecStep:
-                    self.tout = self.tecStep
-                    print(
-                        "Output time interval and tectonic forcing time step \
-                         have been adjusted to match each others.",
-                        flush=True,
-                    )
-                elif self.tout < self.tecStep:
-                    self.tecStep = self.tout
-                    print(
-                        "Output time interval and tectonic forcing time step \
-                         have been adjusted to match each others.",
-                        flush=True,
-                    )
                 try:
-                    self.forceNb = fpaleoDict["freq"]
+                    forceNb = fpaleoDict["steps"]
                 except KeyError:
                     print(
-                        "New Paleomap loading frequency 'freq' is required", flush=True,
+                        "New Paleomap loading frequency 'steps' is required",
+                        flush=True,
                     )
-                out_nb = self.forceNb + 1
-                stepf = np.arange(1, out_nb, dtype=int)
-                self.stepb = np.flip(np.arange(0, out_nb - 1, dtype=int))
-                self.alpha = stepf.astype(float) / (out_nb - 1)
+                outNb = np.sum(forceNb)
+                self.stepb = np.flip(np.arange(0, outNb, dtype=int))
+                self.alpha = np.zeros(len(self.stepb))
+                p = 0
+                for k in range(len(forceNb)):
+                    out_nb = forceNb[k] + 1
+                    stepf = np.arange(1, out_nb, dtype=int)
+                    self.alpha[p : p + len(stepf)] = stepf.astype(float) / (out_nb - 1)
+                    p += len(stepf)
                 self.forceStep = 0
+
+                if self.rStep > 0:
+                    steptime = np.arange(0, outNb, dtype=int) * self.tout + self.tStart
+                    id = np.where(steptime == self.tStart + self.rStep * self.tout)[0]
+                    if len(id) == 0:
+                        raise steptime(
+                            "Something went wrong with the restart time, it needs to be related to the forcing step."
+                        )
+                    self.forceStep = id[0]
+
             except Exception:
                 print(
                     "A directory is required to force the model with paleodata.",
@@ -765,7 +893,53 @@ class ReadYaml(object):
         except KeyError:
             self.forceDir = None
             self.forceStep = -1
-            self.forceNb = 0
+
+        return
+
+    def _readCompaction(self):
+        """
+        Read compaction parameters.
+        """
+
+        try:
+            compDict = self.input["compaction"]
+            try:
+                self.phi0s = compDict["phis"]
+            except KeyError:
+                self.phi0s = 0.49
+
+            try:
+                self.phi0f = compDict["phif"]
+            except KeyError:
+                self.phi0f = 0.63
+
+            try:
+                self.phi0c = compDict["phic"]
+            except KeyError:
+                self.phi0c = 0.65
+
+            try:
+                self.z0s = compDict["z0s"]
+            except KeyError:
+                self.z0s = 3700.0
+
+            try:
+                self.z0f = compDict["z0f"]
+            except KeyError:
+                self.z0f = 1960.0
+
+            try:
+                self.z0c = compDict["z0c"]
+            except KeyError:
+                self.z0c = 2570.0
+
+        except KeyError:
+            self.phi0s = 0.49
+            self.phi0f = 0.63
+            self.phi0c = 0.65
+            self.z0s = 3700.0
+            self.z0f = 1960.0
+            self.z0c = 2570.0
 
         return
 
